Option Strict On

' Used to define the collection object
Public Interface ITreeViewAdapter
    Enum NodeType
        Folder
        Node
    End Enum
    ReadOnly Property Type() As NodeType
    Property OpenIcon() As String
    Property ClosedIcon() As String
    ReadOnly Property Label() As String
    ReadOnly Property ToolTip() As String
    ReadOnly Property item(ByVal index As Integer) As ITreeViewAdapter
    Function Count() As Integer
End Interface

' Used to define the children objects
Public Interface IListViewAdaptor
    Default ReadOnly Property Item(ByVal index As Integer) As IListItem
End Interface

Public Interface IListItem
    Property Icon() As String
    Property ColumnName() As String
    Property ColumnWidth() As Single
    Property Alignment() As String
    Property Sorting() As String
    Property Text() As String
End Interface

Public Class OrderTreeViewAdapter
    Inherits BaseTreeViewAdapter

    Private m_objOrder As Sales.Order

    Sub New(ByVal order As Sales.Order)
        MyBase.New()
        m_objOrder = order

        Dim objLineItem As Sales.LineItem
        For Each objLineItem In m_objOrder.OrderDetails
            Me.Add(New OrderDetailsTreeViewAdapter(objLineItem))
        Next
    End Sub
    Private Sub New()

    End Sub
    Public Overrides ReadOnly Property Type() As ITreeViewAdapter.NodeType
        Get
            Return ITreeViewAdapter.NodeType.Node
        End Get
    End Property

    Public Overrides ReadOnly Property Label() As String
        Get
            Return m_objOrder.CustomerID & " (" & m_objOrder.OrderDetails.Count & ")"
        End Get
    End Property
End Class
Public Class OrderDetailsTreeViewAdapter
    Inherits BaseTreeViewAdapter

    Dim m_objLineItem As Sales.LineItem
    Sub New(ByVal LineItem As Sales.LineItem)
        MyBase.New()

        m_objLineItem = LineItem
    End Sub
    Private Sub New()

    End Sub
    Public Overrides ReadOnly Property Type() As ITreeViewAdapter.NodeType
        Get
            Return ITreeViewAdapter.NodeType.Node
        End Get
    End Property

    Public Overrides ReadOnly Property Label() As String
        Get
            Return m_objLineItem.ProductID & " (" & m_objLineItem.UnitPrice & ")"
        End Get
    End Property
End Class

<Serializable()> Public MustInherit Class BaseTreeViewAdapter
    Inherits System.Collections.CollectionBase
    Implements ITreeViewAdapter

    Private m_strOpenIcon As String
    Private m_strClosedIcon As String

    ' This class implements a strongly typed custom collection
    Public Function Add(ByVal lineItem As ITreeViewAdapter) As Integer
        Return MyBase.List.Add(lineItem)
    End Function
    Public Function Add() As ITreeViewAdapter
        ' No item supplied so will create a new one
        ' Return Me.Item(Me.Add(New ITreeViewAdapter()))
    End Function
    Public Sub Remove(ByVal index As Integer)
        ' Check to see if there is a widget at the supplied index.
        If index > Count() - 1 Or index < 0 Then
            ' If no widget exists, a messagebox is shown and the operation is 
            ' cancelled.
            Throw New Exception("Index not valid!")
        Else
            ' Invokes the RemoveAt method of the List object.
            List.RemoveAt(index)
        End If
    End Sub

    Default Public ReadOnly Property Item(ByVal index As Integer) As ITreeViewAdapter Implements ITreeViewAdapter.item
        Get
            Return CType(List.Item(index), ITreeViewAdapter)
        End Get
    End Property

    Public Property ClosedIcon() As String Implements ITreeViewAdapter.ClosedIcon
        Get
            Return m_strClosedIcon
        End Get
        Set(ByVal Value As String)
            m_strClosedIcon = Value
        End Set
    End Property


    Public Shadows Function Count() As Integer Implements ITreeViewAdapter.Count
        Return MyBase.Count
    End Function

    Public MustOverride ReadOnly Property Label() As String Implements ITreeViewAdapter.Label

    Public Overridable Property OpenIcon() As String Implements ITreeViewAdapter.OpenIcon
        Get
            Return ""
        End Get
        Set(ByVal Value As String)
            Throw New Exception("Must Override property")
        End Set
    End Property

    Public Overridable ReadOnly Property ToolTip() As String Implements ITreeViewAdapter.ToolTip
        Get
            Return ""
        End Get
    End Property

    Public Overridable ReadOnly Property Type() As ITreeViewAdapter.NodeType Implements ITreeViewAdapter.Type
        Get
            Throw New Exception("Must Override property")
        End Get
    End Property
End Class