Option Strict On

Imports System.EnterpriseServices
Imports System.Reflection

' Supply the COM+ application name. 
<Assembly: ApplicationName("SalesOrders")> 
' Supply a COM+ Type.
<Assembly: ApplicationActivation(ActivationOption.Library)> 
' Define where the strong name Key pair is
<Assembly: AssemblyKeyFileAttribute("KeyPair.snk")> 


Namespace Sales.COMPlus
    <Transaction(TransactionOption.Required)> Public Class OrderUpdate
        Inherits ServicedComponent


        ' This method will take an existing order and use the data to update
        ' the database
        Sub Update(ByVal order As Sales.Order)
            Const conConnectionString As String = "server=(local);database=NorthWind;uid=sa;pwd="
            Dim conDatabase As SqlClient.SqlConnection
            Dim cmdCommand As SqlClient.SqlCommand
            Dim intOrderID As Integer

            Try
                ' Do your database and other work here
                ' This example will only deal with New orders
                Debug.WriteLine(ContextUtil.TransactionId())
                If order.OrderID = 0 Then
                    ' We have a new order
                    conDatabase = New SqlClient.SqlConnection(conConnectionString)
                    cmdCommand = New SqlClient.SqlCommand()

                    ' First we create the Order Entry
                    With cmdCommand
                        .CommandType = CommandType.StoredProcedure
                        .CommandText = "usp_Order_Insert"
                        .Parameters.Add("@CustomerID", SqlDbType.NChar).Value _
                            = order.CustomerID
                        .Connection = conDatabase
                        .Connection.Open()
                        intOrderID = CType(.ExecuteScalar, Integer)
                    End With

                    ' Next we create the OrderDetail Entries
                    Dim objLineItem As Sales.LineItem
                    For Each objLineItem In order.OrderDetails
                        ' Ensure we have a fresh Command Object each time
                        cmdCommand = New SqlClient.SqlCommand()
                        With cmdCommand
                            .Connection = conDatabase ' Still open from before
                            .CommandType = CommandType.StoredProcedure
                            .CommandText = "usp_OrderDetails_Insert"
                            .Parameters.Add("@OrderID", SqlDbType.Int).Value _
                                = intOrderID
                            .Parameters.Add("@ProductID", SqlDbType.Int).Value _
                                = objLineItem.ProductID
                            .Parameters.Add("@UnitPrice", SqlDbType.Money).Value _
                                = objLineItem.UnitPrice
                            .Parameters.Add("@Quantity", SqlDbType.SmallInt).Value _
                                = objLineItem.Quantity
                            .Parameters.Add("@Discount", SqlDbType.Real).Value _
                                = objLineItem.Discount
                            If .ExecuteNonQuery() = 0 Then
                                ' No records were added for some reason, so its an ERROR!
                                Throw New Exception("Unable to Insert Record")
                            End If
                        End With
                    Next
                Else
                    Throw New Exception("Existing Order updates are not supported yet.")
                End If
                ' Now tell COM+ the transaction completed successfully
                ContextUtil.SetComplete()
                'ContextUtil.MyTransactionVote = TransactionVote.Commit
            Catch ex As Exception
                ' Do error handling here if required

                ' Now tell COM+ the transaction failed
                'ContextUtil.MyTransactionVote = TransactionVote.Abort
                ContextUtil.SetAbort()
                Throw New Exception("Unable to save Order:" & order.CustomerID)
            Finally
                'Check if the connection object is open and if so close it!
                If conDatabase.State = ConnectionState.Open Then
                    conDatabase.Close()
                End If
            End Try
        End Sub
    End Class
End Namespace
