Option Strict On

Imports System.Messaging
Imports Sales

Public Class Example6a
    Shared Sub Main()
        ' Example 2: Sending a full object as a message
        Dim objMessageQueue As MessageQueue
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.LocalNonTransQueue)
            ' Create an instance of the simple Order Class and populate
            Dim objOrder As New Order()
            objOrder.CustomerID = "ALFKI"
            With objOrder.OrderDetails.Add()
                .ProductID = 1
                .Quantity = 99
                .UnitPrice = 17.5
                .Discount = 0
            End With
            With objOrder.OrderDetails.Add()
                .ProductID = 2
                .Quantity = 20
                .UnitPrice = 18
                .Discount = 0
            End With

            Dim objMessage As New Message()
            With objMessage
                .Body = objOrder
                .Priority = MessagePriority.Highest
                .AdministrationQueue = New MessageQueue(Constants.MQ.LocalAckQueue)
                .AcknowledgeType = AcknowledgeTypes.FullReachQueue Or AcknowledgeTypes.FullReceive
                ' We want them to hang around between computer restarts
                .Recoverable = True
            End With

            ' Send Message as XML
            objMessage.Formatter = New XmlMessageFormatter()
            objMessage.Label = "XML - Order:" & objOrder.CustomerID
            objMessageQueue.Send(objMessage)
            MessageBox.Show("Message Sent")
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
            Stop
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class

' Receiving Ack details.
Public Class Example6b
    Shared Sub Main()
        Dim objMessageQueue As MessageQueue
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.LocalAckQueue)
            Dim objMessage As Message
            ' This is the property which identifies the message
            objMessageQueue.MessageReadPropertyFilter.CorrelationId = True

            ' We will wait only 5 seconds for the Ack to arrive.
            objMessage = objMessageQueue.Receive(New TimeSpan(0, 0, 5))
            If objMessage.MessageType = MessageType.Acknowledgment Then
                MessageBox.Show("The following Ack was received: " & objMessage.Acknowledgment.ToString & " by " & objMessage.CorrelationId.ToString)
            End If
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
    Shared Sub DebugPropertyFilter(ByVal objMessageQueue As MessageQueue)
        With objMessageQueue.MessageReadPropertyFilter
            Debug.WriteLine("AcknowledgeType: " & vbTab & .AcknowledgeType)
            Debug.WriteLine("Acknowledgment: " & vbTab & .Acknowledgment)
            Debug.WriteLine("AdministrationQueue: " & vbTab & .AdministrationQueue)
            Debug.WriteLine("AppSpecific: " & vbTab & .AppSpecific)
            Debug.WriteLine("ArrivedTime: " & vbTab & .ArrivedTime)
            Debug.WriteLine("AttachSenderId: " & vbTab & .AttachSenderId)
            Debug.WriteLine("Authenticated: " & vbTab & .Authenticated)
            Debug.WriteLine("AuthenticationProviderName: " & vbTab & .AuthenticationProviderName)
            Debug.WriteLine("AuthenticationProviderType: " & vbTab & .AuthenticationProviderType)
            Debug.WriteLine("Body: " & vbTab & .Body)
            Debug.WriteLine("ConnectorType: " & vbTab & .ConnectorType)
            Debug.WriteLine("CorrelationId: " & vbTab & .CorrelationId)
            Debug.WriteLine("DefaultBodySize: " & vbTab & .DefaultBodySize)
            Debug.WriteLine("DefaultExtensionSize: " & vbTab & .DefaultExtensionSize)
            Debug.WriteLine("DefaultLabelSize: " & vbTab & .DefaultLabelSize)
            Debug.WriteLine("DestinationQueue: " & vbTab & .DestinationQueue)
            Debug.WriteLine("DestinationSymmetricKey: " & vbTab & .DestinationSymmetricKey)
            Debug.WriteLine("DigitalSignature: " & vbTab & .DigitalSignature)
            Debug.WriteLine("EncryptionAlgorithm: " & vbTab & .EncryptionAlgorithm)
            Debug.WriteLine("Extension: " & vbTab & .Extension)
            Debug.WriteLine("HashAlgorithm: " & vbTab & .HashAlgorithm)
            Debug.WriteLine("Id: " & vbTab & .Id)
            Debug.WriteLine("IsFirstInTransaction: " & vbTab & .IsFirstInTransaction)
            Debug.WriteLine("IsLastInTransaction: " & vbTab & .IsLastInTransaction)
            Debug.WriteLine("Label: " & vbTab & .Label)
            Debug.WriteLine("MessageType: " & vbTab & .MessageType)
            Debug.WriteLine("Priority: " & vbTab & .Priority)
            Debug.WriteLine("Recoverable: " & vbTab & .Recoverable)
            Debug.WriteLine("ResponseQueue: " & vbTab & .ResponseQueue)
            Debug.WriteLine("SenderCertificate: " & vbTab & .SenderCertificate)
            Debug.WriteLine("SenderId: " & vbTab & .SenderId)
            Debug.WriteLine("SenderVersion: " & vbTab & .SenderVersion)
            Debug.WriteLine("SentTime: " & vbTab & .SentTime)
            Debug.WriteLine("SourceMachine: " & vbTab & .SourceMachine)
            Debug.WriteLine("TimeToBeReceived: " & vbTab & .TimeToBeReceived)
            Debug.WriteLine("TimeToReachQueue: " & vbTab & .TimeToReachQueue)
            Debug.WriteLine("TransactionId: " & vbTab & .TransactionId)
            Debug.WriteLine("TransactionStatusQueue: " & vbTab & .TransactionStatusQueue)
            Debug.WriteLine("UseAuthentication: " & vbTab & .UseAuthentication)
            Debug.WriteLine("UseDeadLetterQueue: " & vbTab & .UseDeadLetterQueue)
            Debug.WriteLine("UseEncryption: " & vbTab & .UseEncryption)
            Debug.WriteLine("UseJournalQueue: " & vbTab & .UseJournalQueue)
            Debug.WriteLine("UseTracing: " & vbTab & .UseTracing)

        End With
    End Sub
End Class


Public Class Example6c
    Shared Sub Main()
        Dim objMessageQueue As MessageQueue
        Try
            ' Retreiving an object from a message queue
            objMessageQueue = New MessageQueue(Constants.MQ.LocalNonTransQueue)
            objMessageQueue.Formatter = New XmlMessageFormatter(New Type() {GetType(Order)})
            Dim objMessage As Message = objMessageQueue.Receive(New TimeSpan(0, 0, 10))
            Dim objOrder As Order = DirectCast(objMessage.Body, Order)
            MessageBox.Show("(Message: " & objMessage.Label & ") The customer (" & objOrder.CustomerID & "): has ordered " & objOrder.OrderDetails.Count & " products.")
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            objMessageQueue.Close()
        End Try
    End Sub

End Class
Public Class Example6x
    Shared Sub Main()
        Dim objMessageQueue As MessageQueue
        Try
            ' Retreiving an object from a message queue
            objMessageQueue = New MessageQueue(Constants.MQ.LocalNonTransQueue)
            objMessageQueue.Formatter = New XmlMessageFormatter(New Type() {GetType(String)})
            Dim objMessage As Message = objMessageQueue.Receive(New TimeSpan(0, 0, 10))
            Dim strOrder As String = DirectCast(objMessage.Body, String)
            MessageBox.Show(strOrder)
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class