Option Strict On

Imports System.Messaging
Imports Sales

Public Class Example8a
    Shared Sub Main()
        Dim objMessageQueue As MessageQueue
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.LocalNonTransQueue)
            Dim objMessage As Message

            objMessageQueue.UseJournalQueue = True
            ' We will wait only 5 seconds for the Ack to arrive.
            objMessage = objMessageQueue.Receive(New TimeSpan(0, 0, 5))
            MessageBox.Show("You have mail:")
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            objMessageQueue.Close()
        End Try
    End Sub

End Class

Public Class Example8b
    Shared Sub Main()
        ' Example 2: Sending a full object as a message
        Dim objMessageQueue As MessageQueue
        Try
            objMessageQueue = New MessageQueue(Constants.MQ.PublicNonTransQueue)
            ' Create an instance of the simple Order Class and populate
            Dim objOrder As New Order()
            objOrder.CustomerID = "ALFKI"
            With objOrder.OrderDetails.Add()
                .ProductID = 1
                .Quantity = 99
                .UnitPrice = 17.5
                .Discount = 0
            End With
            With objOrder.OrderDetails.Add()
                .ProductID = 2
                .Quantity = 20
                .UnitPrice = 18
                .Discount = 0
            End With

            Dim objMessage As New Message()
            With objMessage
                .Body = objOrder
                .Priority = MessagePriority.Highest
                .AdministrationQueue = New MessageQueue(Constants.MQ.LocalAckQueue)
                .AcknowledgeType = AcknowledgeTypes.FullReceive
                ' We want them to hang around between computer restarts
                .Recoverable = True
                ' This message must be received within 5 seconds
                .TimeToBeReceived = New TimeSpan(0, 0, 60)
                ' Make a copy of any messages we send
                .UseJournalQueue = True
            End With

            ' Send Message as XML
            objMessage.Formatter = New XmlMessageFormatter()
            objMessage.Label = "(Example8b)XML - Order" & objOrder.CustomerID
            objMessageQueue.Send(objMessage)
            MessageBox.Show("Message Sent")
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class

Public Class Example8c
    Shared Sub Main()
        Dim objMessageQueue As MessageQueue
        Try
            ' Retreiving an object from a message queue
            objMessageQueue = New MessageQueue(Constants.MQ.PublicNonTransQueue)
            objMessageQueue.Formatter = New XmlMessageFormatter(New Type() {GetType(Order)})
            Dim objMessage As Message = objMessageQueue.Receive(New TimeSpan(0, 0, 10))
            Dim objOrder As Order = DirectCast(objMessage.Body, Order)
            MessageBox.Show("(Message: " & objMessage.Label & ") The customer (" & objOrder.CustomerID & "): has ordered " & objOrder.OrderDetails.Count & " products.")
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            objMessageQueue.Close()
        End Try
    End Sub
End Class
Public Class Example8d
    Shared Sub Main()
        Dim objAckQueue As MessageQueue
        Dim objMQ As MessageQueue
        Try
            objAckQueue = New MessageQueue(Constants.MQ.LocalAckQueue)
            Dim objMessage As Message

            objAckQueue.UseJournalQueue = True
            ' If we want to resend we need this information
            objAckQueue.MessageReadPropertyFilter.DestinationQueue = True
            ' We will wait only 30 seconds for the Ack to arrive.
            objMessage = objAckQueue.Receive(New TimeSpan(0, 0, 30))
            Select Case objMessage.Acknowledgment
                Case Acknowledgment.ReceiveTimeout
                    ' We will ask if we want to resend the message
                    If MessageBox.Show("The message: " & objMessage.Label & _
                        " just timed out, do you want to resend?", "Time out", _
                        MessageBoxButtons.YesNo) = DialogResult.Yes Then

                        ' Set the destination of the new message to that of the old.
                        objMQ = New MessageQueue(objMessage.DestinationQueue.FormatName)
                        ' Now send the duplicate message
                        objMQ.Send(copyMessage(objMessage))
                    End If
                Case Else
                    MessageBox.Show("Ack: " & objMessage.Acknowledgment.ToString & " arrived")
            End Select
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        Finally
            objAckQueue.Close()
            objMQ.Close()
        End Try
    End Sub

    Shared Function copyMessage(ByVal originalMessage As Message) As Message
        Dim objNewMessage As New Message()
        ' Use the BinaryReader to easily read the BodyStream object
        Dim objReader As New System.IO.BinaryReader(originalMessage.BodyStream)
        Dim intBytes As Long

        Try
            ' We need to covert the Long to an Integer as the Reader only supports Integers
            ' This is enough for us as an Integer can now handle numbers greater than 4 Million 
            Dim intLength As Integer = CType(originalMessage.BodyStream.Length, Integer)
            ' Copy one stream to another
            objNewMessage.BodyStream.Write(objReader.ReadBytes(intLength), 0, intLength)
            With objNewMessage
                .AdministrationQueue = New MessageQueue(Constants.MQ.LocalAckQueue)
                .AcknowledgeType = AcknowledgeTypes.FullReceive Or AcknowledgeTypes.FullReachQueue
                .TimeToBeReceived = New TimeSpan(0, 0, 40)
                .TimeToReachQueue = New TimeSpan(0, 0, 10)
                .UseJournalQueue = True
                .Label = originalMessage.Label & "(Copy)"
            End With
            Return objNewMessage
        Finally
            ' Ensure we close any resources used
            objReader.Close()
        End Try
    End Function
End Class